#!/usr/bin/perl -w
#
# partitionMetric.pl v1.2.1
# Last modified September 14, 2007 12:01
# (c) Olaf R.P. Bininda-Emonds
#
# Input: A nexus- or phylip-formatted treefile contain one or more trees
#
# Output: A tab-delimited file isting partition metric and normalized partition metric for each
#		  pair of trees. If verbose is specified, output is not divided into columns, but text
#		  blocks for each comparison.
#
# Calculates:
#   The partition metric (number of unique clades / partitions) for two trees pruned to their
#	common taxon sets. Value is given both as the number of clades (rooted trees) or partitions
#	(unrooted trees) found on one tree or the other, but not both and a value normalized for the
#	size of the tree. Will optionally calculate the value weighted according to a measure of
#	nodal support (given as a branch length) and assumed to scale between 0 and 100.
#
# Requires:
#	Perl subroutines dotPlotter, pruneTree, treeRead (all included)
#
# Usage: partitionMetric.pl -t<filename> [-c<a|p|r>] [-r] [-w] [-v] [-h]
#   options: -c<a|p|r> = calculate parition metrix between adjacent pairs of trees (a), between all pairs
#						 of trees (p -- default) or only with respect to first, reference tree (r)
#			 -r = source trees to be rooted unless specified otherwise in treefile
#				  (default = unrooted)
#   		 -t<filename> = treefile containing trees in NEXUS or phylip formats
#			 -w = calculate weighted partition metric using branch lengths as support
#				  values (if applicable); assumes support values range from 0 to 100
#            -v = verbose output
#            -h = print this message and quit

use strict;
use File::Basename;
use constant PI => 3.1415926536;

# Global variables
	# For partition metric
		my $compType = "pairwise";	# Options are "adjacent", (all) "pairwise", and "ref" (all vs. first tree)
		my $weighted_dS = 0;	# Flag as to whether or not to calculate dS weighted by nodal support; assumes latter range from 0-100 (e.g., as for bootstrap frequencies)
		my (%compType, %dSlist, %norm_dSlist);
		my (%pairedDiffList, %pairedTlist, %pairedPlist);

	# Tree variables
		my (%treeName, %treeDescription, %rootingStatus);
		my %prunedTree;
		my $unrootedFlag = 0;
		my $defaultRooting = "[&U]";	# Default is for trees to be unrooted
		my $SupportFlag = 0;
		my @allTaxa;
		my $treeNum;
		
	# Taxon variables
		my (%taxonID, %taxonList);
		my %blankPlot;
		my %compSize;
		
	# Dot plot variables
		my (%dotPlots, %plotWeights);
	
	# I/O parameters
		my $block = 1;
			my $treeBlock = 0;
		my ($treeFile, $resultsFile);

	# Miscellaneous
		my $verbose = 0;
		my $debug = 0;
		my $version = "1.2.1";

# Process switches
	for (my $i = 0; $i <= $#ARGV; $i++)
		{
		if ($ARGV[$i] eq "-ca")
			{
			$compType = "adjacent";
			}
		elsif ($ARGV[$i] eq "-cp")
			{
			$compType = "pairwise";
			}
		elsif ($ARGV[$i] eq "-cr")
			{
			$compType = "ref";
			}
		elsif ($ARGV[$i] eq "-r")
			{
			$defaultRooting = "[&R]";
			}
		elsif ($ARGV[$i] eq "-w")
			{
			$weighted_dS = 1;
			}
		elsif ($ARGV[$i] =~ /-t([\w+\.?\w+?]*)/)
			{
			$treeFile = $1;
			die "ERROR: Treefile $treeFile does not exist.\n" unless (-e $treeFile);
			my ($dataSource, $dir, $ext) = fileparse($treeFile, qr/\.\D.*/);
			$resultsFile = $dataSource ."_dSresults.txt";
			}
		elsif ($ARGV[$i] =~ /-v/)
			{
			$verbose = 1;
			}
		elsif ($ARGV[$i] =~ /-x/)
			{
			$debug = $verbose = 1;
			}
		elsif ($ARGV[$i] eq "-h")
			{
			print "Usage: partitionMetric.pl -t<filename> [-c<a|p|r>] [-r] [-w] [-v] [-h]\n";
			print "Version: $version\n";
			print "Options: -c<a|p|r> = calculate parition metrix between adjacent pairs of trees (a), between all pairs\n";
			print "                     of trees (p -- default) or only with respect to first, reference tree (r)\n";
			print "         -r = source trees to be rooted unless specified otherwise in treefile\n";
			print "              (default = unrooted)\n";
			print "         -t<filename> = file containing trees in NEXUS or phylip formats\n";
			print "         -w = calculate weighted partition metric using branch lengths as support\n";
			print "              values (if applicable); assumes support values range from 0 to 100\n";
			print "         -v = verbose output\n";
			print "         -h = print this message and quit\n";
			exit(0);
			}
		else
			{
			print "Don't understand argument: $ARGV[$i]\n";
			print "Usage: partitionMetric.pl -t<filename> [-c<a|p|r>] [-r] [-w] [-v] [-h]\n";
			print "Version: $version\n";
			exit(1); 
			}
		}

# Check for input data file and process
	die "ERROR: Must supply name of nexus-formatted treefile.\n" if (not $treeFile);

# Read in and store trees
	print "Reading in trees ...\n" if ($verbose);
	my @treeList = treeRead($treeFile);
		die "ERROR: Less than two trees read in from treefile $treeFile\n" if (scalar(@treeList) < 2);
	
	if ($verbose)
		{
		foreach my $treeID (@treeList)
			{
			print "ID: $treeID\n";
			print "\tName: $treeName{$treeID}\n";
			print "\tRooting: ";
				print "un" if ($rootingStatus{$treeID} eq "[&U]");
				print "rooted\n";
			print "\tDescription: $treeDescription{$treeID}\n\n";
			}
		}
		
# Process taxon list and assign IDs
	if ($verbose)
		{
		printf "The following %s taxa were found in the tree set:\n", scalar(@allTaxa);
		}

	for (my $ID = 0; $ID < scalar(@allTaxa); $ID++)
		{
		$taxonID{$allTaxa[$ID]} = $ID;
		print "\t$allTaxa[$ID] ($ID)\n" if ($verbose);
		}
	print "\n" if ($verbose);

# Derive dot plots and taxon lists for each tree
	foreach my $treeID (@treeList)
		{
		# Get taxon list
			(my $taxonString = $treeDescription{$treeID}) =~ s/:\d+\.?\d*e?\-?\d*//ig;	# Remove branch length information
				# Remove any (simple) node labels
					$taxonString =~ s/\)\'*\d+\.*\d*\'*//g;	# Numbers
					$taxonString =~ s/\)\'*\w+\'*//g;	# Words
				$taxonString =~ s/\(//g;
				$taxonString =~ s/\)//g;

			@ { $taxonList{$treeID} } = sort(split(/,/, $taxonString));
			
			# Derive blank dot plot
				$blankPlot{$treeID} = "?" x scalar(@allTaxa);
				substr($blankPlot{$treeID}, $taxonID{$_}, 1, ".") foreach (@ { $taxonList{$treeID} });

		# Get dot plots
			dotPlotterRF($treeID);
		}
		
# Calculate partition metric and output results	
	if ($verbose)
		{
		print "Calculating ";
		print "weighted " if ($weighted_dS);
		print "partition metric ...\n\n";
		}

	my $RFtime = time;
	partitionMetric(@treeList);
		printf "\n\tTotal time taken: %s seconds\n", time - $RFtime if ($verbose);
		my $finalTree = scalar(@treeList - 2);
			$finalTree = 0 if ($compType eq "ref");
			
	print "Printing results to file $resultsFile ...\n" if ($verbose);
	open (OUT, ">$resultsFile") or die "Cannot write results to file $resultsFile\n";
		unless ($verbose)
			{
			print OUT "Number\tTree 1 ID\tTree 2 ID\tComparison type\tShared taxa\t";
				print OUT "Weighted " if ($weighted_dS);
				print OUT "dS\tNormalized ";
				print OUT "weighted " if ($weighted_dS);
				print OUT "dS";
				print OUT "\tPaired difference\tPaired-t\tPaired P" if ($weighted_dS);
			print OUT "\n";
			}
		my $compNum = 0;
		for (my $treeCount1 = 0; $treeCount1 <= $finalTree; $treeCount1++)
			{
			my $tree1 = $treeList[$treeCount1];
			for (my $treeCount2 = $treeCount1 + 1; $treeCount2 < scalar(@treeList); $treeCount2++)
				{
				my $tree2 = $treeList[$treeCount2];
				$compNum++;
				if ($verbose)
					{
					print OUT "Comparison $compNum:\n";
					print OUT "\t$treeName{$tree1} vs. $treeName{$tree2} ($compType{$tree1}{$tree2})\n";
					print OUT "\t\tNumber of shared taxa: $compSize{$tree1}{$tree2}\n";
					print OUT "\t\t";
						print OUT "Weighted " if ($weighted_dS);
						print OUT "dS = $dSlist{$tree1}{$tree2}\n";
					print OUT "\t\tNormalized ";
						print OUT "weighted " if ($weighted_dS);
						print OUT "dS = $norm_dSlist{$tree1}{$tree2}\n";
					if ($weighted_dS and $dSlist{$tree1}{$tree2} eq "0")
						{
						print OUT "\t\tPaired difference = $pairedDiffList{$tree1}{$tree2}";
						printf OUT " (paired t = %.3f; P = %.3f)", $pairedTlist{$tree1}{$tree2}, $pairedPlist{$tree1}{$tree2} unless ($pairedDiffList{$tree1}{$tree2} eq "n/a");
						print OUT "\n";
						}
					}
				else
					{
					print OUT "$compNum\t$treeName{$tree1}\t$treeName{$tree2}\t$compType{$tree1}{$tree2}\t$compSize{$tree1}{$tree2}\t$dSlist{$tree1}{$tree2}\t$norm_dSlist{$tree1}{$tree2}";
					if ($weighted_dS and $dSlist{$tree1}{$tree2} eq "0")
						{
						printf OUT "\t%.3f\t%.3f\t%.3f", $pairedDiffList{$tree1}{$tree2}, $pairedTlist{$tree1}{$tree2}, $pairedPlist{$tree1}{$tree2} unless ($pairedDiffList{$tree1}{$tree2} eq "n/a");
						}
					print OUT "\n";
					}
				last if ($compType eq "adjacent");
				}
			}
	close OUT;

exit(0);

# Subroutines used in script

sub partitionMetric
	{
	my @treeID = @_;
	my $finalTree = scalar(@treeID - 2);
		$finalTree = 0 if ($compType eq "ref");
	my ($dS, $norm_dS);
	
	if ($debug)
		{
		print "Calculating ";
		print "weighted " if ($weighted_dS);
		print "partition metrics ...\n\n";
		}
	
	for (my $treeCount1 = 0; $treeCount1 <= $finalTree; $treeCount1++)
		{
		my $treeID1 = $treeID[$treeCount1];
		my $tree1 = $treeDescription{$treeID1};
		$rootingStatus{$treeID1} = $defaultRooting if (not defined $rootingStatus{$treeID1});
		print "Comparing versus tree $treeCount1 ($treeID1)\n" if ($verbose);
			print "\t$tree1\n\n" if ($debug);
		
		for (my $treeCount2 = $treeCount1 + 1; $treeCount2 < scalar(@treeID); $treeCount2++)
			{
			my $repZero = time;
			
			my $treeID2 = $treeID[$treeCount2];
			my $tree2 = $treeDescription{$treeID2};
			$rootingStatus{$treeID2} = $defaultRooting if (not defined $rootingStatus{$treeID2});
			print "\tComparing versus tree $treeCount2 ($treeID2)\n" if ($verbose);
				print "\t\t$tree2\n\n" if ($debug);
			
			$dS = $norm_dS = 0;
			
			# Determine rooting status for comparison
				$compType{$treeID1}{$treeID2} = "unrooted";
					$compType{$treeID1}{$treeID2} = "rooted" if ($rootingStatus{$treeID1} eq "[&R]" and $rootingStatus{$treeID2} eq "[&R]");
					print "\t\tComparison type: $compType{$treeID1}{$treeID2}\n" if ($verbose);
			
			# Determine intersection and difference of taxon lists
				my (@taxonUnion, @taxonIntersection, @taxonDifference);
			    my %taxonCount;
			    
			    foreach my $element (@ { $taxonList{$treeID1}}, @ { $taxonList{$treeID2}})
			    	{
			    	$taxonCount{$element}++;
			    	}
				foreach my $element (keys %taxonCount)
					{
					push @taxonUnion, $element;
					push @{ $taxonCount{$element} > 1 ? \@taxonIntersection : \@taxonDifference }, $element;
					}
					
				$compSize{$treeID1}{$treeID2} = scalar(@taxonIntersection);
					@taxonIntersection = sort(@taxonIntersection);

			# Determine partition metric
				my (@plotUnion, @plotIntersection, @plotDifference);

				if (($compSize{$treeID1}{$treeID2} < 4 and $compType{$treeID1}{$treeID2} eq "unrooted") or ($compSize{$treeID1}{$treeID2} < 3 and $compType{$treeID1}{$treeID2} eq "rooted"))	# Not enough shared taxa for a meaningful comparison
					{
					$dS = $norm_dS = $dSlist{$treeID1}{$treeID2} = $norm_dSlist{$treeID1}{$treeID2} = "n/a";
					}
				elsif ($compType{$treeID1}{$treeID2} eq "rooted" and scalar(@taxonDifference) == 0)	# Can do simple comparison of dot plots; no post-processing needed
					{
					my %plotCount;
					
					foreach my $element (@ { $dotPlots{$treeID1}}, @ { $dotPlots{$treeID2}})
						{
						$plotCount{$element}++;
						}
					foreach my $element (keys %plotCount)
						{
						push @plotUnion, $element;
						push @{ $plotCount{$element} > 1 ? \@plotIntersection : \@plotDifference }, $element;
						}
						
					foreach my $diffPlot (@plotDifference)
						{
						$dS += $plotWeights{$treeID1}{$diffPlot} if (defined $plotWeights{$treeID1}{$diffPlot});
						$dS += $plotWeights{$treeID2}{$diffPlot} if (defined $plotWeights{$treeID2}{$diffPlot});
						}
					}
				else	# Need to post-process dot plots to account for unrooted comparisons and/or different taxon sets
					{
					my (%plotPresent1, %plotPresent2);
					my (%workingSize, %workingWeight);
					my (@dotPlots1, @dotPlots2);

					my $refTaxon = $taxonIntersection[0];
						print "\t\tReference taxon: $refTaxon ($taxonID{$refTaxon})\n" if ($debug);
					
					# Postprocess dot plots for each tree
						print "\nDot plots from tree $treeName{$treeID1} ...\n" if ($debug);
						foreach my $tree1Plot (@ { $dotPlots{$treeID1}})
							{
							my $workingPlot = $tree1Plot;
							my $plotSize = ($workingPlot =~ tr/\*//);
							
							substr($workingPlot, $taxonID{$_}, 1, "?") foreach (@taxonDifference);	# Correct for different taxon sets by changing unique taxa to ? in plots)
#								$workingPlot =~ s/\?//g if ($debug);

							$workingPlot = invertPlot($workingPlot) if (substr($workingPlot, $taxonID{$refTaxon}, 1) eq "*" and $compType{$treeID1}{$treeID2} eq "unrooted");
								$plotPresent1{$workingPlot}++;
								
							# Check that dot plot remains informative
								my $starCount = ($workingPlot =~ tr/\*//);
								my $dotCount = ($workingPlot =~ tr/\.//);
									next unless ($starCount and $dotCount);
									next if ($starCount <= 1);	# Rooted case: require at least two stars
									next if ($compType{$treeID1}{$treeID2} eq "unrooted" and ($starCount <= 1 or $dotCount <= 1));	# Unrooted case: require at least two
								
							if ($plotPresent1{$workingPlot} == 1)	# First incidence; store weights and size
								{
								$workingSize{$workingPlot} = $plotSize;
								$workingWeight{$workingPlot} = $plotWeights{$treeID1}{$workingPlot} = $plotWeights{$treeID1}{$tree1Plot};
								push @dotPlots1, $workingPlot;
								print "$workingPlot ($plotPresent1{$workingPlot}; weight = $workingWeight{$workingPlot}; $starCount / $dotCount)\n" if ($debug);
								}
							else	# Dot plot replicated due to pruning process
								{
								if ($plotSize < $workingSize{$workingPlot})	# Store weights for least inclusive clade
									{
									$workingSize{$workingPlot} = $plotSize;
									$workingWeight{$workingPlot} = $plotWeights{$treeID1}{$tree1Plot};
									print "$workingPlot ($plotPresent1{$workingPlot}; new weight = $workingWeight{$workingPlot}; $starCount / $dotCount)\n" if ($debug);
									}
								}
							}
	
						print "\nDot plots from tree $treeName{$treeID2} ...\n" if ($debug);
						foreach my $tree2Plot (@ { $dotPlots{$treeID2}})
							{
							my $workingPlot = $tree2Plot;
							my $plotSize = ($workingPlot =~ tr/\*//);
							
							substr($workingPlot, $taxonID{$_}, 1, "?") foreach (@taxonDifference);	# Correct for different taxon sets by changing unique taxa to ? in plots)
#								$workingPlot =~ s/\?//g if ($debug);
							
							$workingPlot = invertPlot($workingPlot) if (substr($workingPlot, $taxonID{$refTaxon}, 1) eq "*" and $compType{$treeID1}{$treeID2} eq "unrooted");
								$plotPresent2{$workingPlot}++;
								
							# Check that dot plot remains informative
								my $starCount = ($workingPlot =~ tr/\*//);
								my $dotCount = ($workingPlot =~ tr/\.//);
									next unless ($starCount and $dotCount);
									next if ($starCount <= 1);	# Rooted case: require at least two stars
									next if ($compType{$treeID1}{$treeID2} eq "unrooted" and ($starCount <= 1 or $dotCount <= 1));	# Unrooted case: require at least two stars and two dots
								
							if ($plotPresent2{$workingPlot} == 1)	# First incidence; store weights and size
								{
								$workingSize{$workingPlot} = $plotSize;
								$workingWeight{$workingPlot} = $plotWeights{$treeID2}{$workingPlot} = $plotWeights{$treeID2}{$tree2Plot};
									 
								push @dotPlots2, $workingPlot;
								if ($debug)
									{
									print "$workingPlot ($plotPresent2{$workingPlot}; weight = $workingWeight{$workingPlot}; $starCount / $dotCount)";
									if (defined $plotPresent1{$workingPlot})
										{
										print ": shared\n";
										}
									else
										{
										print ": unique\n";
										}
									}
								}
							else	# Dot plot replicated due to pruning process
								{
								if ($plotSize < $workingSize{$workingPlot})	# Store weights for least inclusive clade
									{
									$workingSize{$workingPlot} = $plotSize;
									$workingWeight{$workingPlot} = $plotWeights{$treeID2}{$workingPlot} = $plotWeights{$treeID2}{$tree2Plot};
									print "$workingPlot ($plotPresent2{$workingPlot}; new weight = $workingWeight{$workingPlot}); $starCount / $dotCount\n" if ($debug);
									}
								}
							}
							
						# Determine dS from difference in dot plots
							print "\nUnique plots ...\n" if ($debug);
#							my (@plotUnion, @plotIntersection, @plotDifference);
							my %plotCount;
							
							foreach my $element (@dotPlots1, @dotPlots2)
								{
								$plotCount{$element}++;
								}
							foreach my $element (keys %plotCount)
								{
								push @plotUnion, $element;
								push @{ $plotCount{$element} > 1 ? \@plotIntersection : \@plotDifference }, $element;
								}
								
							foreach my $diffPlot (@plotDifference)
								{
								$dS += $workingWeight{$diffPlot};
								print "$diffPlot ($workingWeight{$diffPlot}; dS = $dS)\n" if ($debug);
								}
					}

			unless ($dS eq "n/a")
				{
				# Calculate normalized dS value
					$dS = 0 if ($dS < 10**-5);	# Correction because floating point calculations can occasionally give a slightly non-zero number (i.e., on the order of +/- 1e-16)
					$dSlist{$treeID1}{$treeID2} = $dS;
	
					if ($compType{$treeID1}{$treeID2} eq "rooted")
						{
						$norm_dS = $norm_dSlist{$treeID1}{$treeID2} = sprintf ("%.3f", $dS / ((2 * $compSize{$treeID1}{$treeID2}) - 4));
						}
					else
						{
						$norm_dS = $norm_dSlist{$treeID1}{$treeID2} = sprintf ("%.3f", $dS / ((2 * $compSize{$treeID1}{$treeID2}) - 6));
						}

				# Perform paired t-test if trees topologically identical to see if support values are significantly different
					if ($dS == 0 and $weighted_dS)	# Trees are topologically identical; check
						{
						my ($pairedSum, $pairedAvg, $pairedVar, $pairedSE) = (0, 0, 0, 0);
						my $pairedNum = scalar(@plotIntersection);
						
						if ($pairedNum <= 1)
							{
							$pairedDiffList{$treeID1}{$treeID2} = $pairedTlist{$treeID1}{$treeID2} = $pairedPlist{$treeID1}{$treeID2} = "n/a";
							}
						else
							{
							foreach my $samePlot (@plotIntersection)
								{
								my $supportDiff = $plotWeights{$treeID1}{$samePlot} - $plotWeights{$treeID2}{$samePlot};
									$pairedSum += $supportDiff;
									$pairedVar += $supportDiff * $supportDiff;
								}
		
							# Determine average and variance et al.
								$pairedAvg = $pairedSum / $pairedNum if ($pairedNum > 0);
								$pairedVar = ($pairedVar - ($pairedSum * $pairedSum) / $pairedNum) / ($pairedNum - 1);
									if ($pairedVar <= 0)
										{
										$pairedVar = 0;
										$pairedSE = 0.001;
										}
									else
										{
										$pairedSE = sqrt($pairedVar / $pairedNum);
										}
								
							# Determine if relative rate is significantly different from zero using two-sample t-test
								my $t = $pairedAvg / $pairedSE;
								my $df = $pairedNum - 1;
						
								my $pairedStudentP = _subtprob($df, $t);
									$pairedStudentP = 1 - $pairedStudentP if ($t < 0);
									$pairedStudentP *= 2;	# For two-tailed testing
		
									$pairedDiffList{$treeID1}{$treeID2} = $pairedAvg;
									$pairedTlist{$treeID1}{$treeID2} = $t;
									$pairedPlist{$treeID1}{$treeID2} = $pairedStudentP;
							}

						}
					else
						{
						$pairedDiffList{$treeID1}{$treeID2} = $pairedTlist{$treeID1}{$treeID2} = $pairedPlist{$treeID1}{$treeID2} = "n/a";
						}
				}

			printf "\t\tTime taken: %s second(s)\n", time - $repZero if ($verbose);
			last if ($compType eq "adjacent");
			}
		}
#	return $dS, $norm_dS if (scalar(@treeID) == 2);
	}

sub dotPlotterRF
	{
	my $treeID = shift;
		my $tree = $treeDescription{$treeID};
		print "Tree description:\n\t$tree\n" if ($debug);
	
	my (%ParentClade, %CladeDescrip, %CladeSupport, %open, %start, %stop, %present);
	my %dotPlot;
	
	my $totalClade = ($tree =~ tr/(//);
	my ($position, $CladeID, $CurrentClade) = (0, 0, 0);
	my $OldCurrent;
	
	# Parse tree description and derive dot plots		
		while ($position < length($tree))
			{
			my $offset = 1;
			if (substr($tree, $position, 1) eq "(")		# New clade; go one level deeper
				{
				$CladeID++;
				$ParentClade{$CladeID} = $CurrentClade;
				$CurrentClade = $CladeID;
				$open{$CurrentClade} = 1;
				$start{$CladeID} = $position;
				$CladeSupport{$CladeID} = 1;	# Support defaults to 1 for all clades; only reset as needed
				$dotPlot{$CladeID} = $blankPlot{$treeID};	# Create blank dot plot for clade
				}
			elsif (substr($tree, $position, 1) eq ")")	# Clade finished; clean up and go up one level
				{
				$open{$CurrentClade} = 0;
				$OldCurrent = $CurrentClade;
				$CurrentClade = $ParentClade{$OldCurrent};
				$stop{$OldCurrent} = $position;
				$CladeDescrip{$OldCurrent} = substr($tree, $start{$OldCurrent}, $stop{$OldCurrent}-$start{$OldCurrent}+1);
				$CladeDescrip{$OldCurrent} =~ s/:\d+\.?\d*e?\-?\d*//ig;	# Remove branch length information
				}
			elsif (substr($tree, $position, 1) eq ":")	# Branch lengths present; record as nodal support
				{
				until (substr($tree, $position+$offset, 1) eq "(" or substr($tree, $position+$offset, 1) eq ")" or substr($tree, $position+$offset, 1) eq "," or $position+$offset >= length($tree))
					{
					$offset++;
					}
				if (substr($tree, $position-1, 1) eq ")")	# Support value pertains to a clade; store if desired
					{
					$CladeSupport{$OldCurrent} = sprintf("%.2f", substr($tree, $position+1, $offset-1) / 100) if ($weighted_dS);
					}
				$position += $offset-1;
				}
			elsif (substr($tree, $position, 1) eq ",")
				{
				# Do nothing
				}
			elsif (substr($tree, $position, 1) eq "[")	# Skip over embedded comment
				{
				until (substr($tree, $position+$offset, 1) eq "]" or $position+$offset >= length($tree))
					{
					$offset++;
					}
				$position += $offset-1;
				}
			else	# Determine membership of clade
				{
				my $prevChar = substr($tree, $position-1, 1);
				until (substr($tree, $position+$offset, 1) eq "(" or substr($tree, $position+$offset, 1) eq ")" or substr($tree, $position+$offset, 1) eq "," or substr($tree, $position+$offset, 1) eq ":" or $position+$offset >= length($tree))
					{
					$offset++;
					}
				unless ($prevChar eq ")")	# Text represents a node label and not a taxon
					{
					my $taxon = substr($tree, $position, $offset);
					for (my $clade = 1; $clade <= $CladeID; $clade++)	# Assign taxon to any open clades
						{
						if ($open{$clade})
							{
							$present{substr($tree, $position, $offset)}{$clade} = 1;
							substr($dotPlot{$clade}, $taxonID{$taxon}, 1) = "*";
							}
						}
					}
				$position += $offset-1;
				}
			$position++;
			}

	# Check that dot plots are informative and invert as needed
		my %plotCount;
		for (my $clade = 1; $clade <= $totalClade; $clade++)
			{
			print "Clade $clade:\n\t$CladeDescrip{$clade}\n" if ($debug);
			my $dotPlot = $dotPlot{$clade};

			# Only store informative clades
				my $starCount = ($dotPlot =~ tr/\*//);
					next if ($starCount == 0 or $starCount == length($dotPlot));
					next if ($rootingStatus{$treeID} eq "[&U]" and ($starCount == 1 or $starCount == (length($dotPlot) - 1)));

			# Invert plot to common frame of reference to account for unrooted trees
				my $refTaxon = $taxonList{$treeID}[0];
				$dotPlot = invertPlot($dotPlot) if (substr($dotPlot, $taxonID{$refTaxon}) eq "*" and $rootingStatus{$treeID} eq "[&U]");

			$plotCount{$dotPlot}++;
				print "\tDot plot: $dotPlot\n" if ($debug);
				if ($plotCount{$dotPlot} == 1)	# Only store unique plots
					{
					push @ { $dotPlots{$treeID}}, $dotPlot;
					$plotWeights{$treeID}{$dotPlot} = $CladeSupport{$clade};
					}
				else
					{
					print "\t\tNot unique ...\n" if ($debug);
					}
			}
	}

sub invertPlot	# Change *s to .s (and vice versa) in dot plot
	{
	my $plot = shift;
		$plot =~ s/\./x/g;
		$plot =~ s/\*/\./g;
		$plot =~ s/x/\*/g;

	return $plot;
	}

sub treeRead	# Subroutine to read in tree descriptions and derive taxon names; accounts for any translation tables
	{
	my $treeFile = shift;

	setLineBreak($treeFile);
	open (TREES, "<$treeFile") or die "Cannot open nexus-formatted treefile $treeFile\n";
		my (@allTrees, $rawTreeDescrip, $treeDescrip, $treeFlag, $treeCount, $rooting);
		my (@taxonList, %taxonPresent);
		my (%TTable, $translation, $translateFlag,$maxTaxonID, $transText);
		my $commentFlag = 0;
		$treeBlock = 0;
		
		while (<TREES>)
			{
			chomp;
			$_ =~ s/^\s+//;		# Strip (any) leading white space
			$_ =~ s/\s+$//;		# Strip (any) trailing white space
			
			# Block any commented lines
				if ($_ =~ /^\[/)
					{
					$commentFlag += ($_ =~ tr/\[+//);
					}
				if ($_ =~ /\]\s?$/)
					{
					$_ =~ s/\s+$//;
					$commentFlag -= ($_ =~ tr/\]+//);
					next;
					}
				next if ($commentFlag > 0);
				
			# Process various nexus commands
				if ($_ =~ /^begin trees;/i)		# Clear variables for each tree block
					{
					undef %TTable;
					$translation = $translateFlag = $treeFlag = $treeCount = 0;
					$treeBlock++;
					}
				if ($_ =~ /^translate/i)	# Read translation table of taxon names (if any)
					{
					print "Reading translation table ...\n" if ($verbose);
					$translateFlag = $translation = 1;
					$transText = "";
					$maxTaxonID = 0;
					}
				if ($translateFlag)
					{
					unless ($_ =~ /^translate/i)
						{
						$transText .= $_;
						if ($_ =~ /;/)	# End of translate command; parse
							{
							$translateFlag = 0;
							$transText =~ s/;$//;	#Strip trailing semi-colon
							my @transEntries = split(/,/, $transText);
							foreach my $entry (@transEntries)
								{
								$entry =~ s/^\s+//;		#Strip (any) leading white space
								my @transEntry = split(/\s/,$entry);
									my $taxonID = shift(@transEntry);
									my $taxonName = join(' ', @transEntry);

								$maxTaxonID = $taxonID if ($taxonID > $maxTaxonID);
								$taxonName =~ s/\s+/_/g;
									$taxonName = "'" . $taxonName . "'" if ($taxonName =~ /\W/);
									$taxonName =~ s/^\'+/\'/;
									$taxonName =~ s/\'+$/\'/;
								$TTable{$taxonID} = $taxonName;
								$taxonPresent{$taxonName}++;
								push @taxonList, $taxonName;
								}
							}
						}
					}
				if (($_ =~ /^\s*tree/i and $_ =~ /=/) or $_ =~ /^\(/)
					{
					unless ($_ =~ /^>/)		#Ignore (any) commented lines
						{
						$rawTreeDescrip = $treeDescrip = "";
						unless ($block == 0 and $treeBlock > 1)
							{
							$treeFlag = 1;
							}
						}
					}
	
				if ($treeFlag)
					{
					$rawTreeDescrip .= $_;
					if ($_ =~ /:/)
						{
						$SupportFlag = 1;
						}
					if ($_ =~ /;/)	# End of tree description; parse
						{
						$treeCount++;
						print "\nTree block: $treeBlock\tTree number: $treeCount\n" if ($verbose);
						print "\tTranslating taxon names ...\n" if ($verbose and $translation);
						# Determine if trees are rooted or not (default for no information is not rooted)
							if ($rawTreeDescrip =~ /\[&R\]/)
								{
								$rooting = "[&R]";
								}
							elsif ($rawTreeDescrip =~ /\[&U\]/)
								{
								$rooting = "[&U]";
								$unrootedFlag = 1;
								}
							else
								{
								$rooting = $defaultRooting;
								$unrootedFlag = 1 if ($defaultRooting eq "[&U]");
								}
						# Remove any embedded comments
							if ($rawTreeDescrip =~ /\[/)
								{
								for (my $position = 0; $position < length($rawTreeDescrip); $position++)
									{
									if (substr($rawTreeDescrip, $position, 1) eq "[")
										{
										$position++ until (substr($rawTreeDescrip, $position, 1) eq "]");
										}
									else
										{
										$treeDescrip .= substr($rawTreeDescrip, $position, 1);
										}
									}
								}
							else
								{
								$treeDescrip = $rawTreeDescrip;
								}
						$treeFlag = 0;
						my $position = 0;
						until (substr($treeDescrip, $position, 1) eq "(")		# Avoid anything before tree description proper
							{
							$position++;
							}
						my $tree = substr($treeDescrip, $position);
							$tree =~ s/;\s*$//;
							$tree =~ s/\s+//g;
							
						my $name;
						if ($treeDescrip =~ /=/)
							{
							($name) = (split /=/, $treeDescrip, 0);		# Get name of tree, if any
							$name =~ s/\s*\w*tree\s+//i;
							$name =~ s/\s*\*\s*//i;
							$name =~ s/\s+$//g;
							$name =~ s/-/_/g;
#							$treeNames{$treeCount} = $name;
							}
						my $openCount = ($tree =~ tr/(//);
						my $closedCount = ($tree =~ tr/)//);
						if ($openCount == $closedCount)		# Tree is balanced; process
							{
							$position = length($tree) - 1;
							while ($position >= 0)
								{
								my $IDlength = 1;
								if (substr($tree, $position, 1) eq "(" or substr($tree, $position, 1) eq ")" or substr($tree, $position, 1) eq "," or substr($tree, $position, 1) eq ":")		#Tree structure; skip to next character
									{
									$position--;
									}
								else	# Read in taxonID
									{
									until (substr($tree, $position, 1) eq "(" or substr($tree, $position, 1) eq ")" or substr($tree, $position, 1) eq "," or substr($tree, $position, 1) eq ":")
										{
										$position--;
										$IDlength++;
										}
									unless (substr($tree, $position, 1) eq ":" or substr($tree, $position, 1) eq ")")		# Avoid reading branch lengths or node labels as taxon names
										{
										my $readName = substr($tree, $position+1, $IDlength-1);
										if ($translation and not defined $taxonPresent{$readName})	# Translate taxon names; latter condition is a safety for BioPerl convert_trees.pl module which adds, but doesn't use translation block
											{
											if ($TTable{$readName})
												{
												substr($tree, $position+1, $IDlength-1) = $TTable{$readName};		# Translate name
												unless ($taxonPresent{$TTable{$readName}})
													{
													push @taxonList, $TTable{$readName};
													$taxonPresent{$TTable{$readName}} = 1;
													}
												}
											else
												{
												print "\t\tUnidentified taxon ".$readName." in tree #".$treeCount." of tree block #".$treeBlock.". Replaced with xxx.\n";
												substr($tree, $position+1, $IDlength-1) = "xxx";		# Translate name
												unless ($taxonPresent{"xxx"})
													{
													push @taxonList, "xxx";
													$taxonPresent{"xxx"} = 1;
													}
												}
											}
										else	# Enter name as read from tree, unless already present
											{
											$taxonPresent{$readName}++;
											push @taxonList, $readName if ($taxonPresent{$readName} == 1);
											}
										}
									}
								}
								
							$treeNum++;
								my $treeID = "treeID_$treeNum";
								$name = "readTree_$treeNum" if (not defined $name);
							
							push @allTrees, $treeID;
								$treeDescription{$treeID} = $tree;
								$rootingStatus{$treeID} = $rooting;
								$treeName{$treeID} = $name;
							print "\tTree description:\n\t\t$tree\n" if ($verbose);
							}
						else	#Tree is unbalanced; reject
							{
							print "\tTree #".$treeCount." of tree block #".$treeBlock." is unbalanced. Not processed.\n";
							}
						}
					}
			}
	close TREES;
	
	# Process taxon list
		@allTaxa = sort(duplStrip(@taxonList));
	
	return @allTrees;
	}

sub setLineBreak	# Check line breaks of input files and set input record separator accordingly
	{
	my $inFile = shift;
	$/ ="\n";
	open (IN, "<$inFile") or die "Cannot open $inFile to check form of line breaks.\n";
		while (<IN>)
			{
			if ($_ =~ /\r\n/)
				{
				print "\tDOS line breaks detected ...\n" if ($debug);
				$/ ="\r\n";
				last;
				}
			elsif ($_ =~ /\r/)
				{
				print "\tMac line breaks detected ...\n" if ($debug);
				$/ ="\r";
				last;
				}
			else
				{
				print "\tUnix line breaks detected ...\n" if ($debug);
				$/ ="\n";
				last;
				}
			}
	close IN;
	}

sub duplStrip	# Subroutine to strip duplicate entries from an array
	{
	my @duplArray = @_;
	my %dupl;		# Remove duplicate taxa
		my @stripArray = grep(!$dupl{$_}++, @duplArray);
		@stripArray = sort {$a cmp $b} @stripArray;
	return @stripArray;
	}

sub _subtprob	# Calculates upper probability from Student's t-distribution; taken from the CPAN Perl module Statistics::Distributions by Michael Kospach
	{
	my ($n, $x) = @_;

	my ($a,$b);
	my $w = atan2($x / sqrt($n), 1);
	my $z = cos($w) ** 2;
	my $y = 1;

	for (my $i = $n-2; $i >= 2; $i -= 2)
		{
		$y = 1 + ($i-1) / $i * $z * $y;
		} 

	if ($n % 2 == 0)
		{
		$a = sin($w)/2;
		$b = .5;
		}
	else
		{
		$a = ($n == 1) ? 0 : sin($w)*cos($w)/PI;
		$b= .5 + $w/PI;
		}
	return max(0, 1 - $b - $a * $y);
	}

sub max
	{
	my $max = shift;
	my $next;
	while (@_)
		{
		$next = shift;
		$max = $next if ($next > $max);
		}	
	return $max;
	}

# Version history
#
#	v1.2.1 (September 14, 2007)
#		- added missing max subroutine
#		- fixed bug when calculating paired-t values comparing two topologically identical
#		  trees for the weighted partition metric
#
#	v1.2 (July 3, 2007) (initial release)
#		- retains added features of 1.1, but rewritten for greater speed and efficiency,
#		  including use of new dotPlotterRF module
#		- added ability to calculate if two topologically identical trees have significantly
#		  different support values via a paired Student's t-test (for weighted analyses only)
#		- bug fix to derive taxon names from trees with node labels
#		- minor bug fixes and changes to output format
#
#	v1.1 (May 8, 2006) (never released)
#		- added module pruneTree to ensure that compared trees have same taxon sets
#		- added ability to calculate a weighted dS (by clade support)
#		- added ability to calculate dS between adjacent pairs of trees
#		- now determines dot plots according to rooting status of trees (was always unrooted before) 
#		- added GNU GPL statement
#		- uses treeRead module v1.0.2
#		- updated to much faster dotPlotter / invertPlot modules v1.2
#		- checks that necessary input files exist before proceeding
#		- minor bug fixes and code cleaning
#
#	v1.0 (March 1, 2005)
#		- beta version (in hindsight); never released
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# http://www.gnu.org/copyleft/gpl.html or by writing to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Fifth Floor, Boston, MA, 02110-1301, USA.
